﻿// <copyright company="PEER Group GmbH">
//    Copyright (c) PEER Group GmbH. All rights reserved.
// </copyright>

using System;
using System.Collections.Specialized;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using Infragistics.Windows.DockManager;
using Microsoft.Practices.Prism.Events;
using Microsoft.Practices.Prism.Regions;
using Microsoft.Practices.Prism.Regions.Behaviors;
using Microsoft.Practices.Unity;

namespace SpreadsheetOwnerErrorSampleV3.Core.Prism
{
	/// <summary>
	/// The behavior to support Prism with the XamDockManager. All changes have been applied as per Update 3.
	/// </summary>
	/// <remarks>
	/// Based on source:
	/// http://www.infragistics.com/community/blogs/blagunas/archive/2012/09/13/xamdockmanager-a-prism-regionadapter.aspx
	/// Update 2: http://brianlagunas.com/xamdockmanageran-updated-prism-region-adapter/
	/// update 3: http://brianlagunas.com/xamdockmanager-prism-region-adapter-update-3/
	/// Written by Brian Lagunas.
	/// Blog: http://brianlagunas.com.
	/// Twitter: @brianlagunas.
	/// Email: blagunas@infragistics.com.
	/// </remarks>
	public class TabGroupPaneRegionActiveAwareBehavior : RegionBehavior, IHostAwareRegionBehavior
	{
		internal const string BEHAVIOR_KEY = "TabGroupPaneRegionActiveAwareBehavior";
		
		private TabGroupPane mHostControl;
		private bool mApplicationIsActive;

		/// <summary>Gets or sets the <see cref="DependencyObject"/> that the <see cref="IRegion"/> is attached to.
		/// </summary>
		public DependencyObject HostControl
		{
			get => mHostControl;
			set => mHostControl = value as TabGroupPane;
		}

		/// <summary> Gets or sets the event aggregator. </summary>
		[Dependency]
		internal IEventAggregator EventAggregator { get; set; }

		/// <summary>
		/// Override this method to perform the logic after the behavior has been attached.
		/// </summary>
		protected override void OnAttach()
		{
			const string ERROR_MESSAGE = "ItemsControl's ItemsSource property is not empty. " +
				"This control is being associated with a region, but the control is already bound to something else. " +
				"If you did not explicitly set the control's ItemSource property, this exception may be caused by a " +
				"change in the value of the inherited RegionManager attached property.";

			// If it's data bound we do not allow view injection.
			if (mHostControl.ItemsSource != null)
				throw new InvalidOperationException(ERROR_MESSAGE);

			var dockManager = XamDockManager.GetDockManager(mHostControl);
			if (dockManager != null)
				dockManager.ActivePaneChanged += OnDockManagerActivePaneChanged;

			Region.ActiveViews.CollectionChanged += OnActiveViewsCollectionChanged;
			mApplicationIsActive = true;
			Application.Current.MainWindow.Activated += (s, e) =>
			{
				mApplicationIsActive = true;
			};
			Application.Current.MainWindow.Deactivated += (s, e) =>
			{
				mApplicationIsActive = false;
			};
		}

		/// <summary>Called when the active page changes on the dock manager. </summary>
		/// <param name="sender">The sender.</param>
		/// <param name="e">
		/// The <see cref="RoutedPropertyChangedEventArgs{ContentPane}" /> instance containing the event data.</param>
		private void OnDockManagerActivePaneChanged(object sender, RoutedPropertyChangedEventArgs<ContentPane> e)
		{
			if (!mApplicationIsActive)
				return;

			if (e.OldValue != null)
			{
				var item = e.OldValue;

				// this first checks to see if we had any default views declared in XAML and that were not injected
				if (Region.Views.Contains(item) && Region.ActiveViews.Contains(item))
				{
					Region.Deactivate(item);
				}
				else
				{
					// now check to see if we have any views that were injected
					if (item is ContentControl contentControl)
					{
						var injectedView = contentControl.Content;
						if (Region.Views.Contains(injectedView) && Region.ActiveViews.Contains(injectedView))
							Region.Deactivate(injectedView);
					}
				}
			}

			if (e.NewValue != null)
			{
				var item = e.NewValue;

				// this first checks to see if we had any default views declared in XAML and that were not injected
				if (Region.Views.Contains(item) && !Region.ActiveViews.Contains(item))
				{
					Region.Activate(item);
				}
				else
				{
					// now check to see if we have any views that were injected.
					if (item is ContentControl contentControl)
					{
						var injectedView = contentControl.Content;
						if (Region.Views.Contains(injectedView) && !Region.ActiveViews.Contains(injectedView))
							Region.Activate(injectedView);
					}
				}
			}
		}

		/// <summary>
		/// Called when the active views collection changed. This is responsible for putting the appropriate view into
		/// focus.
		/// </summary>
		/// <param name="sender">The sender.</param>
		/// <param name="e"> The <see cref="NotifyCollectionChangedEventArgs"/> instance containing the event data.
		/// </param>
		private void OnActiveViewsCollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
		{
			if (!mApplicationIsActive)
			{
				return;
			}

			//mLog4View.Debug.Write(messageBuilder.ToString());

			if (!(mHostControl.SelectedItem is ContentPane selectedContentPane))
				return;

			if (e.Action != NotifyCollectionChangedAction.Add)
				return;

			if (selectedContentPane.Content != null)
			{
				var view = selectedContentPane.Content;
				if (view != e.NewItems[0] && Region.ActiveViews.Contains(view))
					Region.Deactivate(view);

				var selectedItem = mHostControl.Items.OfType<ContentPane>()
					.FirstOrDefault(p => p.Content == e.NewItems[0]);
				mHostControl.SelectedItem = selectedItem;
			}

			ContentPane contentPane;

			if (!(e.NewItems[0] is FrameworkElement frameworkElement))
			{
				// Must be a ViewModel
				var viewModel = e.NewItems[0];
				contentPane = GetContentPaneFromViewModel(viewModel);
				contentPane?.Activate();
				return;
			}

			contentPane = frameworkElement as ContentPane ?? frameworkElement.Parent as ContentPane;

			if (!(contentPane is { IsActivePane: false }))
				return;
			
			contentPane.Activate();
		}

		private ContentPane GetContentPaneFromViewModel(object viewModel)
		{
			var panes = XamDockManager.GetDockManager(mHostControl).GetPanes(PaneNavigationOrder.VisibleOrder);
			foreach (var contentPane in panes)
			{
				if (contentPane.DataContext == viewModel)
					return contentPane;
			}

			return null;
		}
	}
}
