﻿// <copyright company="PEER Group GmbH">
//    Copyright (c) PEER Group GmbH. All rights reserved.
// </copyright>

using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using Infragistics.Windows.DockManager;
using Microsoft.Practices.Prism.Regions;
using Microsoft.Practices.Prism.Regions.Behaviors;

namespace SpreadsheetOwnerErrorSampleV3.Core.Prism
{
	/// <summary>
	/// The behavior to support Prism with the XamDockManager on Pane selection changes.
	/// </summary>
	public class TabGroupPaneRegionSelectionChangeBehavior : RegionBehavior, IHostAwareRegionBehavior
	{
		internal const string BEHAVIOR_KEY = "TabGroupPaneRegionSelectionChangeBehavior";

		private TabGroupPane mHostControl;
		
		public DependencyObject HostControl
		{
			get => mHostControl;
			set => mHostControl = value as TabGroupPane;
		}

		/// <summary>
		/// Override this method to perform the logic after the behavior has been attached.
		/// </summary>
		protected override void OnAttach()
		{
			mHostControl.SelectionChanged -= OnSelectionChanged;
			mHostControl.SelectionChanged += OnSelectionChanged;
		}

		/// <summary> Called when the selected view is changed. </summary>
		/// <param name="sender">The sender.</param>
		/// <param name="e"> The <see cref="SelectionChangedEventArgs"/> instance containing
		/// the event data. </param>
		private void OnSelectionChanged(object sender, SelectionChangedEventArgs e)
		{
			var source = e.OriginalSource;
			if (source == sender)
			{
				foreach (var item in e.RemovedItems)
				{
					var contentPane = item as ContentPane;
					if (contentPane == null || contentPane.Content == null)
						continue;

					var view = contentPane.Content;

					// check if the view is in both Views and ActiveViews collections (there may be out of sync)
					if (Region.Views.Contains(view) && Region.ActiveViews.Contains(view))
					{
						Region.Deactivate(view);
					}

					if (e.AddedItems.Count > 0)
						DeselectPane(contentPane);
				}

				foreach (var item in e.AddedItems)
				{
					var contentPane = item as ContentPane;
					if (contentPane?.Content == null)
						continue;

					var view = contentPane.Content;
					if (Region.Views.Contains(view) && !Region.ActiveViews.Contains(view))
					{
						Region.Activate(view);
					}

					SelectPane(contentPane);
				}
			}
		}
		
		private void SelectPane(ContentPane contentPane)
		{
			var dockAware = contentPane as IDockAware ?? contentPane.DataContext as IDockAware;
			if (dockAware != null)
				dockAware.IsSelected = true;

			var resetAware = contentPane as IResetAware ?? contentPane.DataContext as IResetAware;

			if (resetAware != null)
				resetAware.Activate();

			FocusManager.SetFocusedElement(contentPane, contentPane);
		}
		
		private void DeselectPane(ContentPane contentPane)
		{
			var dockAware = contentPane as IDockAware ?? contentPane.DataContext as IDockAware;
			if (dockAware != null)
				dockAware.IsSelected = false;

			var resetAware = contentPane as IResetAware ?? contentPane.DataContext as IResetAware;
			if (resetAware != null)
				resetAware.Deactivate();
		}
	}
}
